@@ -49,7 +49,7 @@ mod terminal;
 mod vscreen;
 pub(crate) mod wrapping;
 
-pub use pretty_printer::{Input, PrettyPrinter};
+pub use pretty_printer::{Input, PrettyPrinter, Syntax};
 pub use syntax_mapping::{MappingTarget, SyntaxMapping};
 pub use wrapping::WrappingMode;
 
