@@ -7,7 +7,7 @@ use globset::{Candidate, GlobBuilder, GlobMatcher};
 
 pub mod ignored_suffixes;
 
-#[derive(Debug, Clone, Copy, PartialEq)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
 #[non_exhaustive]
 pub enum MappingTarget<'a> {
     /// For mapping a path to a specific syntax.
@@ -66,6 +66,14 @@ impl<'a> SyntaxMapping<'a> {
         mapping
             .insert("*.pac", MappingTarget::MapTo("JavaScript (Babel)"))
             .unwrap();
+        mapping
+            .insert("fish_history", MappingTarget::MapTo("YAML"))
+            .unwrap();
+
+        // See #2151, https://nmap.org/book/nse-language.html
+        mapping
+            .insert("*.nse", MappingTarget::MapTo("Lua"))
+            .unwrap();
 
         // See #1008
         mapping
@@ -186,6 +194,7 @@ impl<'a> SyntaxMapping<'a> {
     }
 
     pub(crate) fn get_syntax_for(&self, path: impl AsRef<Path>) -> Option<MappingTarget<'a>> {
+        // Try matching on the file name as-is.
         let candidate = Candidate::new(&path);
         let candidate_filename = path.as_ref().file_name().map(Candidate::new);
         for (ref glob, ref syntax) in self.mappings.iter().rev() {
@@ -197,7 +206,13 @@ impl<'a> SyntaxMapping<'a> {
                 return Some(*syntax);
             }
         }
-        None
+        // Try matching on the file name after removing an ignored suffix.
+        let file_name = path.as_ref().file_name()?;
+        self.ignored_suffixes
+            .try_with_stripped_suffix(file_name, |stripped_file_name| {
+                Ok(self.get_syntax_for(stripped_file_name))
+            })
+            .ok()?
     }
 
     pub fn insert_ignored_suffix(&mut self, suffix: &'a str) {
